
{*******************************************************}
{                                                       }
{       Demo: Multilevel Collapsible sections           }
{                                                       }
{       Copyright (c) Sergey Tkachenko                  }
{       svt@trichview.com                               }
{       http://www.trichview.com                        }
{                                                       }
{*******************************************************}

{ Version 1.1.
  Required TRichView version: 13.4+
}

unit Unit1;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  Dialogs, ImgList, ComCtrls, RVStyle, RVScroll, RichView, RVMarker, RVTypes;

type
  TForm1 = class(TForm)
    PageControl1: TPageControl;
    TabSheet1: TTabSheet;
    TabSheet2: TTabSheet;
    rvDemo: TRichView;
    rvFull: TRichView;
    RVStyle1: TRVStyle;
    ImageList1: TImageList;
    procedure FormCreate(Sender: TObject);
    procedure rvDemoJump(Sender: TObject; id: Integer);
  private
    { Private declarations }
    function IsExpanded(rv: TCustomRichView; ItemNo: Integer): Boolean;
    procedure ChangeExpanded(rv: TCustomRichView; ItemNo: Integer);
    procedure CreateList;
    procedure CreateDoc;
    procedure CopyDoc;
  public
    { Public declarations }

  end;

var
  Form1: TForm1;

implementation

{$R *.dfm}

{ TForm1 }

{
  Creating a document.
  Important:
  - "hotspots" are added just after numbering.
}


{ Creating RVStyle1.ListStyles[0] with multilevel decimal numbering }
procedure TForm1.CreateList;
var i: Integer;
    s: String;
begin
  s := '';
  RVStyle1.ListStyles.Clear;
  with RVStyle1.ListStyles.Add do
    for i := 0 to 9 do
      with Levels.Add do begin
        ListType := rvlstDecimal;
        s := s+'%s.';
        FormatString := s;
        Font.Size := 10;
        Font.Name := 'Comic Sans MS';
        Font.Style := [fsBold];
        Font.Color := clBlue;
        MarkerIndent := i*30;
        LeftIndent := MarkerIndent+20+i*10;
        FirstIndent := 0;
      end;
end;

{ Creating document in rvFull }
procedure TForm1.CreateDoc;

  { Inserting numbering with (optional) hypertext "minus" picture }
  procedure AddNumber(Level: Integer; HasChildren: Boolean);
  begin
     rvFull.SetListMarkerInfo(-1, 0, Level, 0, 0, False);
     if HasChildren then
       rvFull.AddHotspotEx('', 1, 1, ImageList1, -1);
  end;

begin
   CreateList;
   // Building...
   AddNumber(0,True);
   rvFull.Add('Topic', 1);
     AddNumber(1,False);
     rvFull.Add('Bla-bla-bla. Bla-bla-bla. Bla-bla-bla. Bla-bla-bla.', 0);
     AddNumber(1,False);
     rvFull.Add('Bla-bla-bla. Bla-bla-bla. Bla-bla-bla. Bla-bla-bla.', 0);

   AddNumber(0,True);
   rvFull.Add('Topic', 1);
     AddNumber(1,True);
     rvFull.Add('Bla-bla-bla. Bla-bla-bla. Bla-bla-bla. Bla-bla-bla.', 0);
       AddNumber(2,False);
       rvFull.Add('Bla-bla-bla. Bla-bla-bla. Bla-bla-bla. Bla-bla-bla.', 0);
     AddNumber(1,True);
     rvFull.Add('Bla-bla-bla. Bla-bla-bla. Bla-bla-bla. Bla-bla-bla.', 0);
       AddNumber(2,False);
       rvFull.Add('Bla-bla-bla. Bla-bla-bla. Bla-bla-bla. Bla-bla-bla.', 0);
       AddNumber(2,False);
       rvFull.Add('Bla-bla-bla. Bla-bla-bla. Bla-bla-bla. Bla-bla-bla.', 0);

   AddNumber(0,True);
   rvFull.Add('Topic', 1);
     AddNumber(1,False);
     rvFull.Add('Bla-bla-bla. Bla-bla-bla. Bla-bla-bla. Bla-bla-bla.', 0);
     AddNumber(1,False);
     rvFull.Add('Bla-bla-bla. Bla-bla-bla. Bla-bla-bla. Bla-bla-bla.', 0);

   rvFull.Format;
end;
{------------------------------------------------------------------------------}
{ Copying document from rvFull to rvDemo.
  Then removing collapsed sections.
}
procedure TForm1.CopyDoc;
var i,j: Integer;
    Level, StartItemNo: Integer;
begin
  // 1. Copying. AppendFrom can copy all items except for tables and controls.
  rvDemo.Clear;
  rvDemo.AppendFrom(rvFull);
  // 2. Removing collapsed sections
  // Iterating from the last item to the first item...
  for i := rvDemo.ItemCount-1 downto 1 do
    if (rvDemo.GetItemStyle(i)=rvsHotspot) and
       (rvDemo.GetItemStyle(i-1)=rvsListMarker) and
       not IsExpanded(rvDemo, i) then begin // Collapsed paragraph found
      Level := TRVMarkerItemInfo(rvDemo.GetItem(i-1)).Level;
      // Finding the beginning of the next paragraph
      j := i+1;
      while (j<rvDemo.ItemCount) and not rvDemo.IsParaStart(j) do
        inc(j);
      if (j=rvDemo.ItemCount) or
         ((rvDemo.GetItemStyle(j)=rvsListMarker) and
          (TRVMarkerItemInfo(rvDemo.GetItem(j)).Level<=Level)) then
        continue;
      // Finding the next paragraph with the same or higher level (or end of doc)
      StartItemNo := j;
      while (j<rvDemo.ItemCount) and not
         ((rvDemo.GetItemStyle(j)=rvsListMarker) and
          (TRVMarkerItemInfo(rvDemo.GetItem(j)).Level<=Level)) do
        inc(j);
      // Removing all between them
      rvDemo.DeleteItems(StartItemNo, j-StartItemNo);
    end;
  rvDemo.Format;
end;
{------------------------------------------------------------------------------}
procedure TForm1.FormCreate(Sender: TObject);
begin
  CreateDoc;
  CopyDoc;
end;
{------------------------------------------------------------------------------}
{ Clicking hyperlink }
procedure TForm1.rvDemoJump(Sender: TObject; id: Integer);
var ItemNo, i: Integer;
    s: String;
begin
  // GetJumpPointItemNo can be used if you do not use tables (i.e. there are
  // no hyperlinks in table cells
  ItemNo := rvDemo.GetJumpPointItemNo(id);
  if rvDemo.GetItemStyle(ItemNo-1)<>rvsListMarker then
    exit;
  s := TRVMarkerItemInfo(rvDemo.GetItem(ItemNo-1)).DisplayString;
  // Searching rvFull for list marker with the same text
  for i := 0 to rvFull.ItemCount-1 do
    if (rvFull.GetItemStyle(i)=rvsListMarker) and (TRVMarkerItemInfo(rvFull.GetItem(i)).DisplayString=s) then begin
      // Expanding/collasping it
      ChangeExpanded(rvFull, i+1);
      // Copying rvFull to rvDemo, taking expand/collapse state into account
      CopyDoc;
      exit;
    end;
end;
{------------------------------------------------------------------------------}
{ Checking the ItemNo-th item in rv. This item must be a "hotspot".
  If ImageIndex=0, it is "collapsed"
  If ImageIndex=1, it is "expanded"
}
function TForm1.IsExpanded(rv: TCustomRichView; ItemNo: Integer): Boolean;
var ImageIndex1,ImageIndex2: Integer;
    Tag: TRVTag;
    Text: TRVAnsiString;
    il: TCustomImageList;
begin
  rv.GetHotspotInfo(ItemNo, Text, ImageIndex1, ImageIndex2, il, Tag);
  Result := ImageIndex1<>0;
end;
{------------------------------------------------------------------------------}
{ Changing image index the ItemNo-th item in rv (This item must be a "hotspot").
  1 <-> 0
}
procedure TForm1.ChangeExpanded(rv: TCustomRichView; ItemNo: Integer);
var ImageIndex1,ImageIndex2: Integer;
    Tag: TRVTag;
    Text: TRVAnsiString;
    il: TCustomImageList;
begin
  rv.GetHotspotInfo(ItemNo, Text, ImageIndex1, ImageIndex2, il, Tag);
  if ImageIndex1=0 then
    ImageIndex1 := 1
  else
    ImageIndex1 := 0;
  rv.SetHotspotInfo(ItemNo, Text, ImageIndex1, ImageIndex1, il, Tag);
end;

end.
